/**
 * 
 */
package test.DEBBTranslator.src;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author gosiaw
 *
 */
public class PLMXMLTranslator {
	
	private static Log logger = LogFactory.getLog(PLMXMLTranslator.class);
	
	private static final String DEFAULT_PLMXML_TRANSFORMATION_FILE_NAME = "src/test/DEBBTranslator/xml/PLMXMLTranslator.xsl";
	private static final String DEFAULT_UPDATE_TRANSFORMATION_FILE_NAME = "src/test/DEBBTranslator/xml/DCWoRMSDEBBUpdater.xsl";
	private static final String DEFAULT_SCHEDULERS_AND_ESTIMATORS_FILE_NAME = "../../debb2dcworms/example/coolemall1.xml";
	private static final String DEFAULT_TEMPLATE_START_LEVEL = "Node";
	
	private static final String PLMXML_PREFIX = "PLMXML";
	private static final String DCWORMS_PREFIX = "DCWORMS";
	
	private static HashMap<String, String> computingResourceClasses;
	private static HashMap<String, String> resourceUnitClasses;


	/**
	 * 
	 */
	public PLMXMLTranslator() {
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		
		//	To initialize log4j properly
		
		String inputFileName = null;
		String outputFileName = null;
		String debbTransformationFileName = DEFAULT_PLMXML_TRANSFORMATION_FILE_NAME;
		String schedulersAndEstimatorsFileName = DEFAULT_SCHEDULERS_AND_ESTIMATORS_FILE_NAME;
		String templateStartLevel = DEFAULT_TEMPLATE_START_LEVEL;
		
		if (args == null || args.length == 0) {
			String errorMessage = "No arguments specified. Please specify at least input file path.";
			logger.error(errorMessage);
			throw new IOException(errorMessage);
		}
		
		//	Input file
		inputFileName = args[0];
		if (inputFileName == null
				|| (inputFileName != null && inputFileName.isEmpty() == true)) {

			String errorMessage = "Input file not specified!";
			logger.error(errorMessage);
			
			throw new IOException(errorMessage);
		}

		if (args.length > 1) {
			//	Output file
			outputFileName = args[1];
		} 
		if (outputFileName == null
				|| (outputFileName != null && outputFileName.isEmpty() == true)) {
		
			/*if (inputFileName.contains(PLMXML_PREFIX)) {
				//	If file name contains PLMXML replace it by DCWORMS
				outputFileName = inputFileName.replace(PLMXML_PREFIX, DCWORMS_PREFIX);
			} else {*/
				String pathSeparator = File.separator;
//				logger.debug("pathSeparator=" + pathSeparator);	
				
				//	Otherwise add DCWORMS_ prefix
				String path = "";
				int position = inputFileName.lastIndexOf(pathSeparator) ;
				if (position >= 0) {
					path = inputFileName.substring(0, position);
				}
//				logger.debug("path=" + path);	
				
				String name = inputFileName.substring(position + 1);
//				logger.debug("name=" + name);		
				outputFileName = path + "/DCWORMS" + "_" + name;
			//}
			
		}
			
		
		if (args.length > 2) {
			//	File where schedulers and estimators are defined (optional)
			schedulersAndEstimatorsFileName = args[2];
			if (schedulersAndEstimatorsFileName == null
					|| (schedulersAndEstimatorsFileName != null && schedulersAndEstimatorsFileName.isEmpty() == true)) {
				schedulersAndEstimatorsFileName = DEFAULT_SCHEDULERS_AND_ESTIMATORS_FILE_NAME;
				
			}
		}

		if (args.length > 3) {
			//	Level from which all computing resources are defined in templates (optional)
			templateStartLevel = args[3];
			if (templateStartLevel == null
					|| (templateStartLevel != null && templateStartLevel.isEmpty() == true)) {
				templateStartLevel = DEFAULT_TEMPLATE_START_LEVEL;
				
			}
		}
			
		logger.debug("inputFileName=" + inputFileName);
		logger.debug("outputFileName=" + outputFileName);
		logger.debug("schedulersAndEstimatorsFileName=" + schedulersAndEstimatorsFileName);
		logger.debug("templateStartLevel=" + templateStartLevel);
			
		
		if (inputFileName.compareTo(outputFileName) == 0) {
			String errorMessage = "Input and output file cannot be the same";
			logger.error("errorMessage");
			throw new Exception(errorMessage);
		}
		
		// TODO: Check it this is correct!
		// TODO: Move it to configuration file		
		computingResourceClasses = new HashMap<String, String>();
		computingResourceClasses.put("ComputeBox2", "ComputeBox2");
		computingResourceClasses.put("ComputeBox1", "ComputeBox1");
		computingResourceClasses.put("NodeGroup", "NodeGroup");
		computingResourceClasses.put("Node", "Node");
		computingResourceClasses.put("Processor", "Processor");

		resourceUnitClasses = new HashMap<String, String>();
		resourceUnitClasses.put("Sensor", "Sensor");
		resourceUnitClasses.put("Memory", "Memory");
		resourceUnitClasses.put("Baseboard", "Baseboard");
		resourceUnitClasses.put("CoolingDevice", "CoolingDevice");
		resourceUnitClasses.put("Storage", "Storage");
		
		
		try {
			TransformerFactory tFactory = TransformerFactory.newInstance();
						
			Transformer transformer = tFactory
					.newTransformer(new javax.xml.transform.stream.StreamSource(
							debbTransformationFileName));
			transformer.setParameter("computingResourceClasses", computingResourceClasses);
			transformer.setParameter("resourceUnitClasses", resourceUnitClasses);
			transformer.setParameter("templateStartLevel", templateStartLevel);
			
			transformer.transform(new javax.xml.transform.stream.StreamSource(
					inputFileName),
					new javax.xml.transform.stream.StreamResult(
							new FileOutputStream(outputFileName)));
			logger.debug("File " + inputFileName + " transformed into " + outputFileName + " file");
			
			String debbUpdateTransformationFileName = DEFAULT_UPDATE_TRANSFORMATION_FILE_NAME;
			
			Transformer updateTransformer = tFactory.newTransformer(new javax.xml.transform.stream.StreamSource(
							debbUpdateTransformationFileName ));
			updateTransformer.setParameter("additionalInformationFileName", schedulersAndEstimatorsFileName);
			updateTransformer.transform(new javax.xml.transform.stream.StreamSource(outputFileName), 
					new javax.xml.transform.stream.StreamResult(outputFileName));
			logger.debug("File " + outputFileName + " transformed (sorted and updated) into " + outputFileName + " file");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
